import { applyTransforms } from 'svgo/plugins/applyTransforms.js';
import { builtin } from 'svgo/lib/builtin.js';
import { parseSvg } from 'svgo/lib/parser.js';
import { invokePlugins } from 'svgo/lib/svgo/plugins.js';
import { readFile } from 'fs/promises';

const builtinPluginsMap = Object.fromEntries(builtin.map((plugin) => [plugin.name, plugin]));
const MAX_SVG_OPTIMIZATION_ITERATIONS = 10;
const svgoPlugins = [
  { name: "mergeStyles", fn: builtinPluginsMap.mergeStyles.fn },
  { name: "inlineStyles", fn: builtinPluginsMap.inlineStyles.fn },
  { name: "convertStyleToAttrs", fn: builtinPluginsMap.convertStyleToAttrs.fn },
  { name: "removeStyleElement", fn: builtinPluginsMap.removeStyleElement.fn },
  { name: "cleanupNumericValues", fn: builtinPluginsMap.cleanupNumericValues.fn },
  { name: "moveGroupAttrsToElems", fn: builtinPluginsMap.moveGroupAttrsToElems.fn },
  {
    name: "convertShapeToPath",
    params: {
      convertArcs: true
    },
    fn: builtinPluginsMap.convertShapeToPath.fn
  },
  { name: "removeOffCanvasPaths", fn: builtinPluginsMap.removeOffCanvasPaths.fn },
  {
    name: "removeFill",
    fn: () => ({
      element: {
        enter(node) {
          if (node.type !== "element" || node.name !== "path") {
            return;
          }
          delete node.attributes.fill;
        }
      }
    })
  },
  {
    name: "removeClass",
    fn: () => ({
      element: {
        enter(node) {
          if (node.type !== "element" || node.name !== "path") {
            return;
          }
          delete node.attributes.class;
        }
      }
    })
  },
  { name: "applyTransforms", fn: applyTransforms },
  { name: "mergePaths", params: { force: true }, fn: builtinPluginsMap.mergePaths.fn },
  { name: "convertPathData", fn: builtinPluginsMap.convertPathData.fn },
  { name: "collapseGroups", fn: builtinPluginsMap.collapseGroups.fn },
  {
    name: "removeIrrelevantElements",
    fn: () => ({
      element: {
        enter(node, parentNode) {
          if (parentNode.type !== "element" || parentNode.name !== "svg") {
            return;
          }
          if (node.type !== "element" || ["g", "path"].includes(node.name)) {
            return;
          }
          parentNode.children.splice(parentNode.children.indexOf(node), 1);
        }
      }
    })
  }
];
const recursiveSimplifySvg = (ast, plugins, previousIterationSize = Infinity, iterations = 1) => {
  invokePlugins(ast, {}, plugins);
  const size = JSON.stringify(ast).length;
  if (size !== previousIterationSize && iterations >= MAX_SVG_OPTIMIZATION_ITERATIONS) {
    return recursiveSimplifySvg(ast, plugins, size, iterations + 1);
  }
  return ast;
};
const simplifySvg = (svgData, additionalSvgoPlugins) => {
  let plugins;
  if (Array.isArray(additionalSvgoPlugins)) {
    plugins = [...additionalSvgoPlugins, ...svgoPlugins];
  } else if (typeof additionalSvgoPlugins === "object") {
    plugins = [
      ...additionalSvgoPlugins.before ?? [],
      ...svgoPlugins,
      ...additionalSvgoPlugins.after ?? []
    ];
  } else {
    plugins = svgoPlugins;
  }
  return recursiveSimplifySvg(parseSvg(svgData), plugins);
};
const wordNormalizer = (string) => string.split(/[-_.\s]+|(?=(?<![A-Z])[A-Z])/g).map((subString) => subString.toLowerCase().replaceAll(/[^a-z\d\s]/g, ""));
const svgFontawesomeIconLoader = (pluginOptions = {}) => {
  const options = {
    prefix: "cu",
    inferPrefix: false,
    ...pluginOptions
  };
  return {
    name: "svg-fontawesome-icon-loader",
    enforce: "pre",
    async load(moduleId) {
      if (!moduleId.match(/\.svg\?fontawesome/)) {
        return;
      }
      const [filePath, rawQuery] = moduleId.split("?", 2);
      const filename = filePath.split("/").at(-1).slice(0, -4);
      if (filename.length === 0) {
        console.error(
          'vite-svg-fontawesome-icon-loader requires modules to have filenames beyond extension. ".svg" is rejected'
        );
        return;
      }
      let prefix;
      let iconName;
      let aliases;
      if (options.fileNameParser) {
        ({
          prefix = options.prefix,
          iconName,
          aliases = []
        } = options.fileNameParser({
          filename,
          query: rawQuery,
          filePath,
          moduleId
        }));
      } else {
        const query = new URLSearchParams(rawQuery);
        const nameWords = wordNormalizer(filename);
        if (options.inferPrefix && nameWords.length > 1 && nameWords[0].startsWith(options.prefix)) {
          prefix = nameWords[0];
          iconName = nameWords.slice(1).join("-");
        } else {
          prefix = query.get("prefix") ?? options.prefix;
          iconName = query.get("icon-name") ?? nameWords.join("-");
        }
        aliases = query.get("aliases")?.split(",") ?? [];
      }
      const svgFile = await readFile(filePath, "utf-8");
      const optimizedSvgAst = simplifySvg(svgFile, options.svgoPlugins);
      const svgElement = optimizedSvgAst.children.find((node) => node.type === "element" && node.name === "svg");
      if (svgElement === void 0) {
        throw new Error(`Missing svg element in svg file: ${filePath}`);
      }
      const pathData = svgElement.children.filter((node) => node.type === "element" && node.name === "path").map((node) => node.attributes.d) ?? [];
      if (pathData.length === 0) {
        throw new Error(`Couldn't find path or path convertible elements in svg file: ${filePath}`);
      }
      const [
        ,
        ,
        width = svgElement.attributes.width,
        height = svgElement.attributes.height
      ] = svgElement.attributes.viewBox?.split(" ").map(Number) ?? [];
      return `export default ${JSON.stringify({
        prefix,
        iconName,
        icon: [
          width,
          height,
          aliases,
          "",
          pathData.length > 1 ? pathData : pathData[0]
        ]
      }, null, 2)}`;
    }
  };
};

export { svgFontawesomeIconLoader as default };
